import React, { useEffect, useState } from 'react';
import { Add, ArrowDropDown, Delete, Settings } from '@mui/icons-material';
import { DatabaseInfo, useAppDispatch, useDatabaseAPI, useSchemaGraph, useSessionCache } from '@graphpolaris/shared/lib/data-access';
import { updateCurrentDatabase } from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import { SettingsForm } from './forms/settings';
import { NewDatabaseForm } from './forms/AddDatabase/newdatabase';
import { LoadingSpinner } from '@graphpolaris/shared/lib/components/LoadingSpinner';

export default function DatabaseSelector({}) {
  const dispatch = useAppDispatch();
  const api = useDatabaseAPI();
  const session = useSessionCache();
  const schemaGraph = useSchemaGraph();
  const dbSelectionMenuRef = React.useRef<HTMLDivElement>(null);
  const [hovered, setHovered] = useState<string | null>(null);
  const [connecting, setConnecting] = useState<boolean>(false);
  const [dbSelectionMenuOpen, setDbSelectionMenuOpen] = useState<boolean>(false);
  const [settingsMenuOpen, setSettingsMenuOpen] = useState<boolean>(false);
  const [selectedDatabase, setSelectedDatabase] = useState<DatabaseInfo | null>(null);
  const [addDatabaseFormOpen, setAddDatabaseFormOpen] = useState(false);

  useEffect(() => {
    const handleClickOutside = ({ target }: MouseEvent) => {
      if (dbSelectionMenuRef.current && !dbSelectionMenuRef.current.contains(target as Node)) {
        setDbSelectionMenuOpen(false);
      }
    };
    document.addEventListener('click', handleClickOutside);
    return () => {
      document.removeEventListener('click', handleClickOutside);
    };
  }, []);

  useEffect(() => {
    setConnecting(false);
  }, [schemaGraph]);

  return (
    <>
      <SettingsForm
        open={settingsMenuOpen}
        database={selectedDatabase}
        onClose={() => {
          setSettingsMenuOpen(false);
        }}
      />
      <NewDatabaseForm
        open={addDatabaseFormOpen}
        onClose={() => {
          setAddDatabaseFormOpen(false);
        }}
      />
      <div className="relative border w-[20rem] ml-auto mr-auto cursor-pointer" ref={dbSelectionMenuRef}>
        <div
          className="flex w-full justify-between items-center px-4 py-2 hover:bg-slate-200 transition-colors duration-300"
          onClick={() => setDbSelectionMenuOpen(!dbSelectionMenuOpen)}
        >
          <div className="flex items-center w-full">
            {connecting ? (
              <LoadingSpinner />
            ) : session.currentDatabase ? (
              <div className="h-2 w-2 rounded-full bg-green-500" />
            ) : (
              <div className="h-2 w-2 rounded-full bg-slate-500" />
            )}
            {connecting ? (
              <p className="ml-2 truncate">Connecting to {session.currentDatabase}</p>
            ) : session.currentDatabase ? (
              <p className="ml-2 truncate">Connected DB: {session.currentDatabase}</p>
            ) : (
              <p className="ml-2">Select a database</p>
            )}
          </div>
          <ArrowDropDown />
        </div>
        {dbSelectionMenuOpen && (
          <div className="absolute w-full top-11 z-50 bg-slate-100 border">
            <div
              className="flex items-center p-2 hover:bg-slate-200"
              onClick={(e) => {
                e.preventDefault();
                setDbSelectionMenuOpen(false);
                setConnecting(false);
                setAddDatabaseFormOpen(true);
              }}
              title="Add new database"
            >
              <Add />
              <p className="ml-2">Add database</p>
            </div>
            {session.databases.map((db) => (
              <div
                key={db.Name}
                className="flex justify-between items-center px-4 py-2 hover:bg-slate-200"
                onClick={(e) => {
                  if (db.Name !== session.currentDatabase) {
                    e.preventDefault();
                    setDbSelectionMenuOpen(false);
                    setConnecting(true);
                    dispatch(updateCurrentDatabase(db.Name));
                  } else {
                    setDbSelectionMenuOpen(false);
                  }
                }}
                onMouseEnter={() => setHovered(db.Name)}
                onMouseLeave={() => setHovered(null)}
                title={`Connect to ${db.Name}`}
              >
                <div className="flex items-center">
                  <div className={`h-2 w-2 rounded-full mr-2 ${db.status ? 'bg-green-500' : 'bg-red-500'}`} />
                  <div>
                    <p className="ml-2">{db.Name}</p>
                    <p className="ml-2 text-xs text-slate-400">
                      <span className="border border-slate-300 px-1">{db.Protocol}</span> {db.URL}
                    </p>
                  </div>
                </div>
                {hovered === db.Name && (
                  <div className="flex items-center">
                    <div
                      className="text-slate-700 hover:text-slate-400 transition-colors duration-300"
                      onClick={(e) => {
                        e.preventDefault();
                        e.stopPropagation();
                        setSettingsMenuOpen(true);
                        setSelectedDatabase(db);
                      }}
                    >
                      <Settings />
                    </div>
                    <div
                      className="text-slate-700 hover:text-slate-400 transition-colors duration-300"
                      onClick={(e) => {
                        e.preventDefault();
                        dispatch(updateCurrentDatabase(''));
                        api.DeleteDatabase(db.Name);
                      }}
                      title="Delete database"
                    >
                      <Delete />
                    </div>
                  </div>
                )}
              </div>
            ))}
          </div>
        )}
      </div>
    </>
  );
}
